(*
StartupAce StartupForm.pas Unit (startup form's implementation)
Copyright (C) 2006  Jakob Reschke

This software is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this software; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*)

unit StartupForm;

interface

uses
  TBXOffice2003Theme, Menus, TB2Item, TBX, SpTBXItem, ExtCtrls, TBXSwitcher,
  TB2Dock, TB2Toolbar, Classes, Controls, VirtualTrees, Forms, Graphics, 
  Windows, StartupEntry, TBXDkPanels, SpTBXDkPanels, TntExtCtrls, StdCtrls,
  TntStdCtrls, Buttons, TntButtons;

type
  TStartForm = class(TForm)
    SpTBXDock1: TSpTBXDock;
    SpTBXToolbar1: TSpTBXToolbar;
    ExecNowMI: TSpTBXItem;
    CancelMI: TSpTBXItem;
    SkipMI: TSpTBXItem;
    PauseMI: TSpTBXItem;
    SpTBXSeparatorItem1: TSpTBXSeparatorItem;
    SpTBXSeparatorItem2: TSpTBXSeparatorItem;
    ElapsedTimeCaptionLb: TSpTBXLabelItem;
    ElapsedTimeLabel: TSpTBXLabelItem;
    SpTBXStatusBar1: TSpTBXStatusBar;
    TBXSwitcher1: TTBXSwitcher;
    DelayTimer: TTimer;
    ConfigI: TSpTBXItem;
    SpTBXRightAlignSpacerItem1: TSpTBXRightAlignSpacerItem;
    SpTBXPopupMenu1: TSpTBXPopupMenu;
    SkipThisMI: TSpTBXItem;
    ExecNowThisMI: TSpTBXItem;
    SpTBXSeparatorItem3: TSpTBXSeparatorItem;
    ToggleEntriesActivityMI: TSpTBXItem;
    ClientPanel: TPanel;
    VST: TVirtualStringTree;
    ShowHintsBtn: TTntButton;
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure VSTGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType; var CellText: WideString);
    procedure VSTBeforeItemErase(Sender: TBaseVirtualTree;
      TargetCanvas: TCanvas; Node: PVirtualNode; ItemRect: TRect;
      var ItemColor: TColor; var EraseAction: TItemEraseAction);
    procedure VSTCompareNodes(Sender: TBaseVirtualTree; Node1,
      Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
    procedure CancelMIClick(Sender: TObject);
    procedure DelayTimerElapsed(Sender: TObject);
    procedure PauseMIClick(Sender: TObject);
    procedure SkipMIClick(Sender: TObject);
    procedure VSTFreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
    procedure ConfigIClick(Sender: TObject);
    procedure ExecNowThisMIClick(Sender: TObject);
    procedure SkipThisMIClick(Sender: TObject);
    procedure ExecNowMIClick(Sender: TObject);
    procedure VSTKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure VSTPaintText(Sender: TBaseVirtualTree;
      const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
      TextType: TVSTTextType);
    procedure ToggleEntriesActivityMIClick(Sender: TObject);
    procedure SpTBXPopupMenu1Popup(Sender: TObject);
    procedure ShowHintsBtnClick(Sender: TObject);
  strict private
    { fields }
    ElapsedTime: Cardinal;
    FinishedTime: Cardinal;
    FPause,
    FFinished: Boolean;
  protected
    { properties and their setters and getters }
    function GetCurrentEntry: PVirtualNode;                     
    function GetEntryData: PStartupEntry;            
    property CurrentEntryNode: PVirtualNode read GetCurrentEntry;
    property CurrentEntry: PStartupEntry read GetEntryData;
    property Finished: Boolean read FFinished write SetFinished;
  protected
    { setters and getters for public properties }
    procedure SetPause(const Value: Boolean);
  private
    procedure SetFinished(const Value: Boolean);
  protected
    { methods }
    procedure Start(node: PVirtualNode; DeleteNode: Boolean);
    procedure Finished;
    procedure EvaluateEntry(Node: PVirtualNode);
  public
    { public methods }
    procedure Delete(node: PVirtualNode);
  published
  published
  public
    { public properties }
    property Pause: Boolean read FPause write SetPause;
  end;

  TIdleEntriesThread = class(TThread)
    protected
      procedure Execute; override;  
  end;

var
  StartForm: TStartForm;

implementation

uses
  Statics, ShellAPI, StartupAceSettings, SysUtils, StUpEntrysLoader, Math,
  TlHelp32;

{$R *.dfm}

function ExtractFilePathW(FileName: WideString): WideString;
var
  I: Integer;
begin
  for I := Length(FileName) - 1 downto 1 do
  begin
    if copy(FileName, I, 1) = '\' then
    begin
      Result := copy(FileName, 1, I - 1);
      exit;
    end;
  end;
end;

procedure TStartForm.ConfigIClick(Sender: TObject);
begin
  Pause := True;
  ShellExecuteW(handle, 'open', PWChar(WideString(ParamStr(0))), '',
    PWChar(ExtractFilePathW(ParamStr(0))), SW_SHOWNORMAL);
end;

procedure TStartForm.DelayTimerElapsed(Sender: TObject);
var
  e: PStartupEntry;
  n: PVirtualNode;
  label notfinished;
begin
  Inc(ElapsedTime);
  if FFinished then
  begin
    if (PStartupEntry(VST.GetNodeData(VST.GetFirst)).Arguments <> 'Close Startup Ace')
      then
    begin
      FFinished := False;
      goto notfinished;
    end;
    if 10 + FinishedTime - ElapsedTime <= 0 then
    begin
      Close;
    end;
  end
  else
  begin
  notfinished:
    VST.Invalidate;
    ElapsedTimeLabel.Caption := IntToStr(ElapsedTime);
    DelayTimer.Enabled := false;
    n := VST.GetFirst;
    while n <> nil do
    begin
      e := VST.GetNodeData(n);
      if e.Delay - ElapsedTime = 0 then
      begin
        Start(n, false);
        n := VST.GetNext(n);
      end
      else if Integer(e.Delay) - Integer(ElapsedTime) < 0 then
      begin
        if e.WaitFor = '' then
          Delete(n)
        else
          Start(n, false);          
        n := VST.GetNext(n); // TODO: sicher, dass das so hier geht?
      end
      else n := nil;
    end;
    n := VST.GetFirst;
    while (n <> nil)
      and not (PStartupEntry(VST.GetNodeData(n)).Delay - ElapsedTime > 0) do
      n := VST.GetNext(n);
    if n = nil then
      Finished
    else
    begin
      e := VST.GetNodeData(n);
      if e.Delay - ElapsedTime > 0 then
      begin
        if not Pause then
        begin
          DelayTimer.Enabled := true;
        end;
      end;
      if not e.Active then
        Finished;
    end;
  end;
end;

procedure TStartForm.ExecNowMIClick(Sender: TObject);
begin
  Start(VST.GetFirst, true);
end;

procedure TStartForm.ExecNowThisMIClick(Sender: TObject);
begin
  Start(VST.FocusedNode, true);
end;

procedure TStartForm.Finished;
var
  e: TStartupEntry;
begin
  e := TStartupEntry.Create(nil);
  e.Delay := ElapsedTime + 10;
  e.Caption := 'Finished';
  e.Path := 'REM';
  e.Arguments := 'Close Startup Ace';
  e.Active := True;
  e.TreeNode := VST.AddChild(VST.RootNode, @e);
  VST.SortTree(1, sdAscending);
  FFinished := true;
  FinishedTime := ElapsedTime;
  ExecNowMI.Enabled := false;
  SkipMI.Enabled := false;
  PauseMI.Enabled := false;
  ElapsedTimeLabel.Caption := '--';
  DelayTimer.Enabled := true;
end;

procedure TStartForm.FormCreate(Sender: TObject);
begin
  VST.NodeDataSize := sizeof(TStartupEntry);
  TBXSwitcher1.Theme := 'Office2003';
end;

procedure TStartForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    VK_F2: begin
             ConfigIClick(self);
           end;
    Word('p'), Word('P'): begin
                            if ssCtrl in Shift then
                              PauseMIClick(self);
                          end;
  end;
end;

procedure TStartForm.FormPaint(Sender: TObject);
begin
  self.OnPaint := nil;
  StartupEntriesLoader.Load(VST, TStartupEntryEvent(nil),
    StartupEntriesLoader.StartupAce
    or StartupEntriesLoader.CurrentUser
    or StartupEntriesLoader.AllUsers
    or StartupEntriesLoader.Inactive);
  VST.SortTree(1, sdAscending);
  Pause := TStartupAceSettings.StartPaused;
  if VST.GetFirst <> nil then
  begin
    if PStartupEntry(VST.GetNodeData(VST.GetFirst)).Delay = 0 then
    begin
      Start(VST.GetFirst, false);
    end;
    DelayTimer.Enabled := True;
  end
  else
    Finished;
end;

function TStartForm.GetCurrentEntry: PVirtualNode;
begin
  Result := VST.GetFirst;
end;

function TStartForm.GetEntryData: PStartupEntry;
begin
  Result := VST.GetNodeData(CurrentEntryNode);
end;

procedure TStartForm.EvaluateEntry(Node: PVirtualNode);
var
  e: PStartupEntry;
begin
  e := VST.GetNodeData(Node);
  if e.Delay - ElapsedTime <= 0 then
  begin
    Start(Node, false);
    if VST.GetNext(Node) <> nil then
      EvaluateEntry(VST.GetNext(Node));
  end
  else
  begin
    DelayTimer.Enabled := true;
  end;  
end;

procedure TStartForm.CancelMIClick(Sender: TObject);
begin
  Close;
end;

procedure TStartForm.PauseMIClick(Sender: TObject);
begin
  Pause := not Pause;
end;

procedure TStartForm.SetFinished(const Value: Boolean);
begin
  FFinished := Value;
  if FFinished then
  begin
    // TODO: add close startup ace entry
  end
  else
  begin
    // TODO: remove close startup ace entry
  end;
end;

procedure TStartForm.SetPause(const Value: Boolean);
begin
  FPause := Value;
  PauseMI.Checked := FPause;
  if not FPause then
  begin
    DelayTimer.Enabled := true;
  end;
end;

function ProcessRunning(ProcessName: WideString): Boolean;
var
  SSHandle: THandle;
  ProcEntry: TProcessEntry32W;
  Continue: Boolean;
  osvi: OSVERSIONINFO;
begin  
  Result := False;
  osvi.dwOSVersionInfoSize := SizeOf(OSVERSIONINFO);
  GetVersionEx(osvi);
  if (osvi.dwMajorVersion > 4)
    or (osvi.dwPlatformId = VER_PLATFORM_WIN32_WINDOWS) then
  begin
    SSHandle := CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
    try
      ProcEntry.dwSize := SizeOf(ProcEntry);
      Continue := Process32FirstW(SSHandle, ProcEntry);
      while Continue do
      begin
        if WideString(ProcEntry.szExeFile) = ProcessName then
        begin
          Result := True;
          Continue := False;
        end
        else
        begin
          Continue := Process32NextW(SSHandle, ProcEntry);
        end;
      end;
    finally
      CloseHandle(SSHandle);
    end;
  end;
end;

procedure Execute(handle: HWND; e: PStartupEntry);
begin
//  ShellExecuteW(handle, 'open', PWideChar(e.Path),
//    PWideChar(e.Arguments),
//    PWideChar(ExtractFilePathW(e.Path), SW_SHOWNORMAL);
end;

procedure TStartForm.Start(node: PVirtualNode; DeleteNode: Boolean);
var
  e: PStartupEntry;
  label start;
begin
  e := VST.GetNodeData(node);
  if (e.Active) then
  begin
    if e.WaitFor <> '' then
    begin
      if ProcessRunning(e.WaitFor) then
      begin
        Execute(handle, e);
        e.WaitFor := '';
      end;
    end
    else
      Execute(handle, e);
  end;
end;

procedure TStartForm.ToggleEntriesActivityMIClick(Sender: TObject);
var
  e: PStartupEntry;
begin
  e := VST.GetNodeData(VST.FocusedNode);
  e.Active := not e.Active;
  if not e.Active then
  begin
    if not (e.Delay = e.OriginalDelay) then
      e.Delay := e.OriginalDelay;
  end;
  e.Save(false);
  if e.Active and (e.Delay < ElapsedTime) then
    e.Delay := ElapsedTime + e.Delay;
  VST.SortTree(1, sdAscending);
  VST.InvalidateNode(VST.FocusedNode);
end;

procedure TStartForm.ShowHintsBtnClick(Sender: TObject);
var
  r: Cardinal;
  msg: WideString;
begin
  r := ShellExecuteW(handle, nil, 'doc\startupformhints.htm', nil, nil,
    SW_SHOWNORMAL);
  if r <= 32 then
  begin
    case Integer(r) of
      ERROR_FILE_NOT_FOUND: msg := 'File doc\startupformhints.htm not found!';
      SE_ERR_ACCESSDENIED: msg := 'Access to doc\startupformhints.htm denied!';
    end;
    MessageBoxW(handle, PWideChar(msg), 'Error', MB_ICONERROR);
  end;
end;

procedure TStartForm.Delete(node: PVirtualNode);
begin
  if node = VST.GetFirst then
  begin
    VST.DeleteNode(node);
    if VST.GetFirst = nil then
      Finished;
  end
  else
  begin
    VST.DeleteNode(node);
  end;
end;

procedure TStartForm.SkipMIClick(Sender: TObject);
begin
  Delete(VST.GetFirst);
end;

procedure TStartForm.SkipThisMIClick(Sender: TObject);
begin
  Delete(VST.FocusedNode);
end;

procedure TStartForm.SpTBXPopupMenu1Popup(Sender: TObject);
var
  e: PStartupEntry;
begin
  if VST.FocusedNode <> nil then
  begin
    e := VST.GetNodeData(VST.FocusedNode);
    ToggleEntriesActivityMI.Enabled := e.Editable;
    if e.Active then
      ToggleEntriesActivityMI.Caption := 'De&activate this startup entry'
    else
      ToggleEntriesActivityMI.Caption := 'Re&activate this startup entry';
  end;
end;

procedure TStartForm.VSTBeforeItemErase(Sender: TBaseVirtualTree;
  TargetCanvas: TCanvas; Node: PVirtualNode; ItemRect: TRect;
  var ItemColor: TColor; var EraseAction: TItemEraseAction);
var
  e: PStartupEntry;
begin
  e := Sender.GetNodeData(Node);
  if Sender.GetFirst = Node then
  begin
    ItemColor := $E0FFFF;
    EraseAction := eaColor;
  end
  else if (e.Delay - ElapsedTime <= 10) and e.Active then
  begin
    ItemColor := cl3DLight;
    EraseAction := eaColor;
  end;
end;

procedure TStartForm.VSTCompareNodes(Sender: TBaseVirtualTree; Node1,
  Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
begin
  case Column of
    1:
    begin
      if PStartupEntry(Sender.GetNodeData(Node1)).Active then
      begin
        if PStartupEntry(Sender.GetNodeData(Node2)).Active then
        begin
          Result := PStartupEntry(Sender.GetNodeData(Node1))^.Delay -
            PStartupEntry(Sender.GetNodeData(Node2))^.Delay;
        end
        else
        begin
          Result := -1;
        end;
      end
      else
      begin
        if PStartupEntry(Sender.GetNodeData(Node2)).Active then
        begin
          Result := 1;
        end
        else
        begin
          Result := PStartupEntry(Sender.GetNodeData(Node1))^.Delay -
            PStartupEntry(Sender.GetNodeData(Node2))^.Delay;
        end;
      end;
    end;
  end;
end;

procedure TStartForm.VSTFreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
var
  data: PStartupEntry;
begin
  data := Sender.GetNodeData(Node);
  data^.Free;
  data^.Caption := '';
end;

procedure TStartForm.VSTGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
  Column: TColumnIndex; TextType: TVSTTextType; var CellText: WideString);
var
  e: PStartupEntry;
begin        
  e := VST.GetNodeData(Node);
  case Column of
    0: CellText := e.Caption;
    1:
      if e.Delay = e.OriginalDelay then
        CellText := IntToStr(e.Delay)
      else
        CellText := '(' + IntToStr(e.Delay) + ')';
    2:
      if Sender.Selected[Node]
        or ((e.Delay - ElapsedTime <= 10) and e.Active)
        or ((Integer(e.Delay) - Integer(ElapsedTime) < 0) and (e.WaitFor <> ''))
        or (VST.GetFirst = Node)
        then
      begin
        if e.Active then
          if e.Delay - ElapsedTime = 0 then
            CellText := 'Is beeing started...'
          else if (Integer(e.Delay) - Integer(ElapsedTime) < 0 ) and (e.WaitFor <> '') then
            CellText := 'Waiting for process ' + e.WaitFor + ' to start...'
          else
            CellText := IntToStr((e.Delay - ElapsedTime)) + ' seconds'
        else
          CellText := 'Deactivated';
      end
      else CellText := '';
  end;
end;

procedure TStartForm.VSTKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  r: TRect;
  p1: TPoint;
  p2: TPoint;
  n: PVirtualNode;
begin
  case Key of
    VK_SPACE: begin
                if ssShift in Shift then
                  Start(CurrentEntryNode, true)
                else
                  Start(VST.FocusedNode, true);
              end;
    VK_DELETE: begin
                 if VST.FocusedNode <> nil then
                 begin
                  n := VST.GetNext(VST.FocusedNode);
                  if n = nil then
                    n := VST.GetPrevious(VST.FocusedNode);
                  if ssShift in Shift then
                    Delete(CurrentEntryNode)
                  else
                    Delete(VST.FocusedNode);
                  VST.FocusedNode := n;
                  VST.Selected[n] := True;
                  VST.SetFocus;
                 end;
               end;
    VK_RETURN: begin
                 r := VST.GetDisplayRect(VST.FocusedNode, 0, true);
                 p1 := VST.ClientToScreen(r.TopLeft);
                 p2 := VST.ClientToScreen(r.BottomRight);
                 VST.PopupMenu.Popup(p1.X + 10, p2.Y);
               end;
  end;
end;

procedure TStartForm.VSTPaintText(Sender: TBaseVirtualTree;
  const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  TextType: TVSTTextType);
var
  e: PStartupEntry;
  NH: Integer;
  Left: Integer;
begin
  e := Sender.GetNodeData(Node);
  if e.Active then
    TargetCanvas.Font.Color := clWindowText
  else
    TargetCanvas.Font.Color := clGrayText;
  if Sender.Selected[Node] or (e.Delay - ElapsedTime <= 10) and e.Active then
  begin
    Sender.NodeHeight[Node] :=
      (Sender as TVirtualStringTree).DefaultNodeHeight * 2;
    if (e.Delay - ElapsedTime <= 10) and e.Active then
    begin
      NH := Sender.NodeHeight[Node];
      Left := (Sender as TVirtualStringTree).GetDisplayRect(Node, 2, false).Left;
      TargetCanvas.Pen.Color := clInactiveCaption;
      TargetCanvas.Brush.Color := clGradientInactiveCaption;
      if e.Delay - ElapsedTime = 10 then
        TargetCanvas.Ellipse(Left + 2,2,Left + NH-2,NH-2)
      else if e.Delay - ElapsedTime >= 8 then
        TargetCanvas.Pie(Left + 2,2, Left + NH-2,NH-2,
          Left + Ceil(1 + (NH/2 - NH/2 * Tan(Pi - (e.Delay-ElapsedTime)*Pi/5))), 0,
          Left + 1 + NH div 2, 0)
      else if e.Delay - ElapsedTime >= 6 then
        TargetCanvas.Pie(Left + 2,2,Left + NH-2,NH-2,
          Left + Ceil(1 + (NH/2 - NH/2 * Tan((e.Delay-ElapsedTime)*Pi/5 - Pi))), NH,
          Left + 1 + NH div 2, 0)
      else if e.Delay - ElapsedTime = 5 then
        TargetCanvas.Pie(Left + 2,2,Left + NH-2,NH-2,
          Left + 1 + NH div 2, NH,
          Left + 1 + NH div 2, 0)
      else if e.Delay - ElapsedTime >= 3 then
         TargetCanvas.Pie(Left + 2,2,Left + NH-2,NH-2,
          Left + Ceil(1 + (NH/2 + NH/2 * Tan(Pi - (e.Delay-ElapsedTime)*Pi/5))), NH,
          Left + 1 + NH div 2, 0)
      else if e.Delay - ElapsedTime > 0 then
        TargetCanvas.Pie(Left + 2,2,Left + NH-2,NH-2,
          Left + Ceil(1 + (NH/2 + NH/2 * Tan((e.Delay-ElapsedTime)*Pi/5))), 0,
          Left + 1 + NH div 2, 0);
    end;             
  end
  else
  begin
    Sender.NodeHeight[Node] := (Sender as TVirtualStringTree).DefaultNodeHeight;
  end;
end;

{ TIdleEntriesThread }

procedure TIdleEntriesThread.Execute;
begin
  inherited;

end;

end.
