(*
StartupAce MessageStrs.pas Unit
Interface for accessing localized strings in resources\messagestrings.xml file.
Copyright (C) 2007  Jakob Reschke

This software is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this software; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*)

unit MessageStrs;

interface

type
  MessageStrings = class
    protected
      class function GetString(Name: WideString): WideString; static;
    public
      class function RegistryBrowsingNotImplemented: WideString; static;
      class function NameInUse: WideString; static;
      class function NameInUseTitle: WideString; static;
      class function DiscardChanges: WideString; static;
      class function Confirmation: WideString; static;
      class function CannotAccessEntrysRegistryKey: WideString; static;
      class function CannotAccessRunKey: WideString; static;
      class function CannotAccessStartupAceKey: WideString; static;
      class function SaveErrorAbort: WideString; static;
      class function Question: WideString; static;
      class function NameUsedMultipleTimes: WideString; static;
      class function DuplicateName: WideString; static;
      class function ReallySave: WideString; static;
      class function AccessDenied: WideString; static;
  end;

implementation

uses
  xmldom, xmldoc, Windows, SysUtils, Dialogs;

{ MessageStrings }

class function MessageStrings.AccessDenied: WideString;
begin
  Result := GetString('AccessDenied');
end;

class function MessageStrings.CannotAccessEntrysRegistryKey: WideString;
begin
  Result := GetString('CannotAccessEntrysRegistryKey');
end;

class function MessageStrings.CannotAccessRunKey: WideString;
begin
  Result := GetString('CannotAccessRunKey');
end;

class function MessageStrings.CannotAccessStartupAceKey: WideString;
begin
  Result := GetString('CannotAccessStartupAceKey');
end;

class function MessageStrings.Confirmation: WideString;
begin
  Result := GetString('Confirmation');
end;

class function MessageStrings.DiscardChanges: WideString;
begin
  Result := GetString('DiscardChanges');
end;

class function MessageStrings.DuplicateName: WideString;
begin
  Result := GetString('DuplicateName');
end;

class function MessageStrings.GetString(Name: WideString): WideString;
var
  doc: xmldoc.TXMLDocument;
  ELEMENT_BY_ID: Boolean;
  stringNode: IDOMElement;
  strings: IDOMNodeList;
  locals: IDOMNodeList;
  I: Integer;
  buf: array [0..3] of WideChar;
  lang: WideString;
begin
  doc := TXMLDocument.Create('resources\messagestrings.xml');
  try
    ELEMENT_BY_ID := doc.DOMImplementation.hasFeature('getElementById', '1');
    if ELEMENT_BY_ID then
    begin
      stringNode := doc.DOMDocument.getElementById(Name);
      locals := stringNode.getElementsByTagName('local');
    end
    else
    begin
      strings := doc.DOMDocument.getElementsByTagName('string');
      for I := 0 to strings.length - 1 do
        if strings.item[I].attributes.getNamedItem('xml:id').nodeValue = Name then
        begin
          locals := (strings.item[I] as IDOMElement).getElementsByTagName('local');
          break;
        end;
    end;
    if locals = nil then raise Exception.Create('No strings defined for message' +
                                                ' id "' + Name + '"!');
    GetLocaleInfoW(GetUserDefaultLCID, LOCALE_SABBREVLANGNAME, @buf, 4);
    lang := WideString(buf);
    for I := 0 to locals.length - 1 do
      if Uppercase(
        locals.item[I].attributes.getNamedItem('lang').nodeValue) = lang then
      begin
        Result := (locals.item[I] as IDOMElement).firstChild.nodeValue;
        break;
      end
      else if Uppercase(
        locals.item[I].attributes.getNamedItem('lang').nodeValue) = 'ENG' then
        Result := (locals.item[I] as IDOMElement).firstChild.nodeValue;
  finally
    doc.Free;
  end;
end;

class function MessageStrings.NameInUse: WideString;
begin
  Result := GetString('NameInUse');
end;

class function MessageStrings.NameInUseTitle: WideString;
begin
  Result := GetString('NameInUseTitle');
end;

class function MessageStrings.NameUsedMultipleTimes: WideString;
begin
  Result := GetString('NameUsedMultipleTimes');
end;

class function MessageStrings.Question: WideString;
begin
  Result := GetString('Question');
end;

class function MessageStrings.ReallySave: WideString;
begin
  Result := GetString('ReallySave');
end;

class function MessageStrings.RegistryBrowsingNotImplemented: WideString;
begin
  Result := GetString('RegistryBrowsingNotImplemented');
end;

class function MessageStrings.SaveErrorAbort: WideString;
begin
  Result := GetString('SaveErrorAbort');
end;

end.
