(*
StartupAce PathCtrls.pas Unit (contains staic class with methods
for operations on edit controls dealing with file paths)
Copyright (C) 2006  Jakob Reschke

This software is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this software; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*)
unit PathCtrls;

interface

uses
  TntDialogs, SpTBXEditors;

type
  PathControls = class
  protected
    class function InsertPath(var Dest: WideString; const Index: Integer;
      FileSelectionDialog: TTntOpenDialog): Integer; overload; static;
  public
    class procedure InsertPath(EditControl: TSpTbxUnicodeEdit;
      FileSelectionDialog: TTntOpenDialog); overload; static;
    class procedure InsertPath(EditControl: TSpTbxUnicodeEdit); overload; static;
  end;

implementation

{ FilePathEdit }

class function PathControls.InsertPath(var Dest: WideString; const Index: Integer;
  FileSelectionDialog: TTntOpenDialog): Integer;
var
  s: WideString;
begin
  Result := -MaxInt;
  if FileSelectionDialog.Execute then
  begin
    s := '';
    Result := 0;
    if (Index > 0) and (Dest[Index] <> ' ') then
    begin
      s := s + ' ';
      Inc(Result);
    end;
    s := s + '"' + FileSelectionDialog.FileName + '"';
    if (Index+1 <= Length(Dest)) and (Dest[Index+1] <> ' ') then
    begin
      s := s + ' ';
    end;
    Dest := Copy(Dest, 1, Index) + s +
      Copy(Dest, Index + 1, Length(Dest) - Index);
  end;
end;

class procedure PathControls.InsertPath(EditControl: TSpTbxUnicodeEdit;
  FileSelectionDialog: TTntOpenDialog);
var
  s: WideString;
  initialSelStart: Integer;
  offset: Integer;
begin
  s := EditControl.Text;
  initialSelStart := EditControl.SelStart;
  offset := InsertPath(s, EditControl.SelStart, FileSelectionDialog);
  if offset > -MaxInt then
  begin
    EditControl.Text := s;
    EditControl.SelStart := initialSelStart + offset;
    EditControl.SelLength := Length(FileSelectionDialog.FileName) + 2;
  end;
  EditControl.SetFocus;
end;

class procedure PathControls.InsertPath(EditControl: TSpTbxUnicodeEdit);
var
  dlg: TTntOpenDialog;
begin
  dlg := TTntOpenDialog.Create(EditControl);
  with dlg do
  begin
    try
      Filter := 'All files|*.*';
      InsertPath(EditControl, dlg);
    finally
      Free;
    end;
  end;
end;

end.
