(*
StartupAce Policy.pas Unit (information about user's writing privilegies)
Copyright (C) 2006  Jakob Reschke

This software is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this software; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*)

// This is indeed not written by me (Jakob Reschke). Source (German):
// http://www.delphi-forum.de/viewtopic.php?t=2779

unit Policy;

interface

uses Windows;

function IsAdmin: LongBool;

implementation

function GetAdminSid: PSID; 
const 
  // bekannte SIDs ... (WinNT.h) 
  SECURITY_NT_AUTHORITY: TSIDIdentifierAuthority = (Value: (0, 0, 0, 0, 0, 5)); 
  // bekannte RIDs ... (WinNT.h) 
  SECURITY_BUILTIN_DOMAIN_RID: DWORD = $00000020; 
  DOMAIN_ALIAS_RID_ADMINS: DWORD = $00000220; 
begin 
  Result := nil; 
  AllocateAndInitializeSid(SECURITY_NT_AUTHORITY, 2, 
    SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_ADMINS, 
    0, 0, 0, 0, 0, 0, Result); 
end;

function IsAdmin: LongBool; 
var
  TokenHandle: THandle; 
  ReturnLength: DWORD; 
  TokenInformation: PTokenGroups; 
  AdminSid: PSID; 
  Loop: Integer; 
begin 
  Result := False; 
  TokenHandle := 0; 
  if OpenProcessToken(GetCurrentProcess, TOKEN_QUERY, TokenHandle) then 
  try 
    ReturnLength := 0; 
    GetTokenInformation(TokenHandle, TokenGroups, nil, 0, ReturnLength); 
    TokenInformation := GetMemory(ReturnLength); 
    if Assigned(TokenInformation) then 
    try 
      if GetTokenInformation(TokenHandle, TokenGroups, TokenInformation, 
        ReturnLength, ReturnLength) then 
      begin 
        AdminSid := GetAdminSid; 
        for Loop := 0 to TokenInformation^.GroupCount - 1 do 
        begin 
          if EqualSid(TokenInformation^.Groups[Loop].Sid, AdminSid) then 
          begin 
            Result := True; 
            Break; 
          end; 
        end; 
        FreeSid(AdminSid); 
      end; 
    finally 
      FreeMemory(TokenInformation); 
    end; 
  finally 
    CloseHandle(TokenHandle); 
  end; 
end;

end.
