unit StUpEntrysLoader;

interface

uses VirtualTrees, StartupEntry;

type
  StartupEntriesLoader = class
    const Registry = 1;
    const StartupAce = 2;
    const CurrentUser = 4;
    const AllUsers = 8;
    const Inactive = 16;
    const Links = 32;
    class procedure Load(TargetVST: TVirtualStringTree;  
      OnChangedEventHandler: TStartupEntryEvent;
      Flags: Cardinal); overload; static;
  end;

implementation

uses Registry, Classes, Windows, Statics, Forms, SysUtils, ShlObj, ActiveX,
  ComObj, JSysDirs;
       
class procedure StartupEntriesLoader.Load(TargetVST: TVirtualStringTree;
  OnChangedEventHandler: TStartupEntryEvent;
  Flags: Cardinal);
var
  reg: TRegistry;
  values: TStringList;
  keys: TStringList;
  node: PVirtualNode;
  e: PStartupEntry;

  procedure ProcessWinEntries(RootKey: HKEY);
  var
    I: Integer;
  begin
    reg.RootKey := RootKey;
    if reg.OpenKeyReadOnly(RegistryKeys.Run) then
    begin
      try
        reg.GetValueNames(Values);
        for I := 0 to Values.Count - 1 do
        begin
          node := TargetVST.AddChild(nil);
          e := PStartupEntry(TargetVST.GetNodeData(node));
          e^ := TStartupEntry.Create(node);
          e^.OnChanged := OnChangedEventHandler;
          e^.Caption := values[I];
          e^.ParseRegString(reg.ReadString(values[I]));
          if (Flags and Inactive = 0) and not e^.Active then
          begin
            TargetVST.DeleteNode(node);
          end
          else
          begin
            e^.RunType := rtRegistry;
            e^.AllUsers := RootKey <> HKEY_CURRENT_USER;
            e^.Delay := 0;
            e^.SetCurrentAsOriginalValues;
          end;
        end;
      finally
        reg.CloseKey;
      end;
    end
    else
    begin
      MessageBoxW(Application.Handle,
        'Cannot access the Windows startup registry key!',
        'Error',
        MB_OK or MB_ICONERROR);
    end;
  end;

  procedure ProcessStartupAceEntries(RootKey: HKEY);
  var
    I: Integer;
  begin
    reg.RootKey := RootKey;
    if reg.OpenKeyReadOnly(RegistryKeys.StartupAce) then
    begin
      reg.GetKeyNames(keys);
      reg.CloseKey;
      for I := 0 to keys.Count - 1 do
      begin
        if reg.OpenKeyReadOnly(RegistryKeys.StartupAce + '\' + keys[I]) then
        begin
          node := TargetVST.AddChild(nil);
          e := PStartupEntry(TargetVST.GetNodeData(node));
          e^ := TStartupEntry.Create(node);
          e^.Active := StrToBool(reg.ReadString('Active'));
          if (Flags and Inactive = 0) and not e^.Active then
          begin
            TargetVST.DeleteNode(Node);
          end
          else
          begin
            e^.OnChanged := OnChangedEventHandler;
            e^.Caption := keys[I];
            e^.Path := UTF8Decode(reg.ReadString('Path'));
            e^.Arguments := UTF8Decode(reg.ReadString('Parameters'));
            e^.RunType := rtStartupAce;
            e^.Delay := reg.ReadInteger('Delay');
            e^.AllUsers := RootKey <> HKEY_CURRENT_USER;
            if reg.ValueExists('WaitFor') then
              e.WaitFor := UTF8Decode(reg.ReadString('WaitFor'))
            else
              e.WaitFor := '';
            {$region 'Abwrtskompatibilitt'}
            if e^.Path = '' then
              e^.Path := reg.ReadString('Program');
            {$endregion}            
            e^.SetCurrentAsOriginalValues;
          end;
          reg.CloseKey;
        end;
      end;
    end;
    reg.CloseKey;
  end;

  procedure AutostartsFromLinkFiles(Dir: WideString; AllUsers: Boolean);
  var
    sr: TSearchRec;
    path: WideString;
    n: PVirtualNode;
    s: WideString;
    IObj: IUnknown;
    Link: IShellLinkW;
    IPFile: IPersistFile;
    FindDataDummy: TWin32FindDataW;
    buffer: array [0..MAX_PATH] of WChar;
  begin
    IObj := CreateComObject(CLSID_ShellLink);
    Link := IObj as IShellLinkW;
    IPFile := IObj as IPersistFile;
    if Dir[length(dir)] <> '\' then Dir := Dir + '\';
    if FindFirst(Dir + '*.lnk', faAnyFile, sr) = 0 then
    begin
      repeat
        if sr.Attr and faDirectory = 0 then
        begin
          path := Dir + sr.Name;
          if IPFile.Load(PWideChar(path), fmOpenRead) = S_OK then
          begin
            n := TargetVST.AddChild(nil);
            e := TargetVST.GetNodeData(n);
            e^ := TStartupEntry.Create(n);
            e^.Caption := ExtractFileName(path);
            FillChar(buffer, MAX_PATH, #0);
            Link.GetPath(buffer, MAX_PATH, FindDataDummy, 0);
            e^.Active := Uppercase(copy(WideString(buffer), 1, 3)) <> 'REM';
            if e^.Active then
              e^.Path := Trim(WideString(buffer))
            else
            begin
              s := Trim(WideString(buffer));
              s := copy(s, 3, Length(s) - 3);
              e^.Path := TrimLeft(s);
              s := '';
            end;
            FillChar(buffer, MAX_PATH, #0);
            Link.GetArguments(buffer, MAX_PATH);
            e^.Arguments := Trim(WideString(buffer));
            e^.Caption := copy(e^.Caption, 1, Length(e^.Caption) - 4); // remove .lnk
            e^.RunType := rtLnk;
            e^.AllUsers := AllUsers;
            e^.Delay := 0;
            e^.SetCurrentAsOriginalValues;
          end;
        end;
      until FindNext(sr) <> 0;
      FindClose(sr);
    end;
  end;

begin
  reg := TRegistry.Create;
  values := TStringList.Create;
  keys := TStringList.Create;
  try
    TargetVST.Clear;
    if (StartupAce and Flags) > 0 then
    begin
      if (CurrentUser and Flags) > 0 then ProcessStartupAceEntries(HKEY_CURRENT_USER);
      if (AllUsers and Flags) > 0 then ProcessStartupAceEntries(HKEY_LOCAL_MACHINE);
    end;
    if (Registry and Flags) > 0 then
    begin
      if (CurrentUser and Flags) > 0 then ProcessWinEntries(HKEY_CURRENT_USER);
      if (AllUsers and Flags) > 0 then ProcessWinEntries(HKEY_LOCAL_MACHINE);
    end;
    if (Links and Flags) > 0 then
    begin
      if (CurrentUser and Flags) > 0 then AutostartsFromLinkFiles(GetSpecialFolderW(CSIDL_STARTUP), false);
      if (AllUsers and Flags) > 0 then AutostartsFromLinkFiles(GetSpecialFolderW(CSIDL_COMMON_STARTUP), true);
    end;
  finally
    reg.CloseKey;
    reg.Free;
    values.Free;
    keys.Free;
  end;
end;

end.
