(*
StartupAce StartupAceSettings.pas Unit
(static class for persistent program settings)
Copyright (C) 2006, 2007  Jakob Reschke

This software is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this software; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*)

unit StartupAceSettings;

interface

uses
  Statics;

type
  TStartupAceSettings = class
    strict private
    protected
      class function GetStartPaused: Boolean; static;
      class procedure SetStartPaused(Value: Boolean); static;
      class function GetOffice2003Style: Boolean; static;
      class procedure SetOffice2003Style(Value: Boolean); static;
      class function GetStartMinimized: Boolean; static;
      class procedure SetStartMinimized(Value: Boolean); static;
    public              
      class property StartPaused: Boolean read GetStartPaused write SetStartPaused;
      class property Office2003Style: Boolean read GetOffice2003Style write SetOffice2003Style;
      class property StartMinimized: Boolean read GetStartMinimized write SetStartMinimized;
  end;

implementation

uses
  Registry, Windows, Forms;

{ TStartupAceSettings }

class function TStartupAceSettings.GetOffice2003Style: Boolean;
begin
  with TRegistry.Create do
  begin
    try
      RootKey := HKEY_CURRENT_USER;
      if OpenKey(RegistryKeys.StartupAce, false) then
      begin
        if ValueExists('Office2003Style') then
          Result := ReadBool('Office2003Style')
        else
          Result := false;
      end
      else
      begin
        Result := False;
      end;
    finally
      CloseKey;
      Free;
    end;
  end;
end;

class function TStartupAceSettings.GetStartMinimized: Boolean;
begin
  with TRegistry.Create do
  begin
    try
      RootKey := HKEY_CURRENT_USER;
      if OpenKey(RegistryKeys.StartupAce, False) then
      begin
        if ValueExists('StartMinimized') then
          Result := ReadBool('StartMinimized')
        else
          Result := False;
      end
      else
      begin
        Result := False;
      end;
    finally
      CloseKey;
      Free;
    end;
  end;
end;

class function TStartupAceSettings.GetStartPaused: Boolean;
begin
  with TRegistry.Create do
  begin
    try
      RootKey := HKEY_CURRENT_USER;
      if OpenKey(RegistryKeys.StartupAce, false) then
      begin
        if ValueExists('StartPaused') then
          Result := ReadBool('StartPaused')
        else
          Result := false;
      end
      else
      begin
        Result := False;
      end;
    finally
      CloseKey;
      Free;
    end;
  end;
end;

class procedure TStartupAceSettings.SetOffice2003Style(Value: Boolean);
begin
  with TRegistry.Create do
  begin
    try
      RootKey := HKEY_CURRENT_USER;
      if OpenKey(RegistryKeys.StartupAce, True) then
        WriteBool('Office2003Style', Value)
      else
        MessageBoxW(Application.Handle, 'Could not open/create the Startup Ace' +
                                        'registry key!' +
                                        'An "Office 2003 Style" setting was not saved!',
                                        'Access error', MB_ICONERROR); 
    finally
      CloseKey;
      Free;
    end;
  end;
end;

class procedure TStartupAceSettings.SetStartMinimized(Value: Boolean);
begin
  with TRegistry.Create do
  begin
    try
      RootKey := HKEY_CURRENT_USER;
      if OpenKey(RegistryKeys.StartupAce, True) then
        WriteBool('StartMinimized', Value)
      else
      MessageBoxW(Application.Handle, 'Could not open/create the Startup Ace' +
                                      ' registry key!' +
                                      'The "Start minimized" setting has not ' +
                                      'been saved!',
                                      'Access error', MB_ICONERROR);
    finally
      CloseKey;
      Free;
    end;
  end;
end;

class procedure TStartupAceSettings.SetStartPaused(Value: Boolean);
begin
  with TRegistry.Create do
  begin
    try
      RootKey := HKEY_CURRENT_USER;
      if OpenKey(RegistryKeys.StartupAce, true) then
        WriteBool('StartPaused', Value)
      else
        MessageBoxW(Application.Handle, 'Could not open/create the Startup Ace' +
                                        'registry key!' +
                                        'A "Start in paused mode" setting was not saved!',
                                        'Access error', MB_ICONERROR); 
    finally
      CloseKey;
      Free;
    end;
  end;
end;

end.
