(*
StartupAce StartupForm.pas Unit (startup form's implementation)
Copyright (C) 2006  Jakob Reschke

This software is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this software; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*)

unit StartupForm;

interface

uses
  TBXOffice2003Theme, Menus, TB2Item, TBX, SpTBXItem, ExtCtrls, TBXSwitcher,
  TB2Dock, TB2Toolbar, Classes, Controls, VirtualTrees, Forms, Graphics, 
  Windows, StartupEntry, TBXDkPanels, SpTBXDkPanels, TntExtCtrls, StdCtrls,
  TntStdCtrls, Buttons, TntButtons;

type
  TStartForm = class(TForm)
    SpTBXDock1: TSpTBXDock;
    SpTBXToolbar1: TSpTBXToolbar;
    ExecNowMI: TSpTBXItem;
    CancelMI: TSpTBXItem;
    SkipMI: TSpTBXItem;
    PauseMI: TSpTBXItem;
    SpTBXSeparatorItem1: TSpTBXSeparatorItem;
    SpTBXSeparatorItem2: TSpTBXSeparatorItem;
    SpTBXStatusBar1: TSpTBXStatusBar;
    TBXSwitcher1: TTBXSwitcher;
    DelayTimer: TTimer;
    SpTBXPopupMenu1: TSpTBXPopupMenu;
    SkipThisMI: TSpTBXItem;
    ExecNowThisMI: TSpTBXItem;
    SpTBXSeparatorItem3: TSpTBXSeparatorItem;
    ToggleEntriesActivityMI: TSpTBXItem;
    ClientPanel: TPanel;
    VST: TVirtualStringTree;
    ElapsedTimeCaptionLb: TSpTBXLabelItem;
    ElapsedTimeLabel: TSpTBXLabelItem;
    ConfigI: TSpTBXItem;
    ShowHintsBtn: TSpTBXItem;
    procedure FormCreate(Sender: TObject);
    procedure VSTGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType; var CellText: WideString);
    procedure VSTBeforeItemErase(Sender: TBaseVirtualTree;
      TargetCanvas: TCanvas; Node: PVirtualNode; ItemRect: TRect;
      var ItemColor: TColor; var EraseAction: TItemEraseAction);
    procedure VSTCompareNodes(Sender: TBaseVirtualTree; Node1,
      Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
    procedure CancelMIClick(Sender: TObject);
    procedure DelayTimerElapsed(Sender: TObject);
    procedure PauseMIClick(Sender: TObject);
    procedure SkipMIClick(Sender: TObject);
    procedure VSTFreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
    procedure ConfigIClick(Sender: TObject);
    procedure ExecNowThisMIClick(Sender: TObject);
    procedure SkipThisMIClick(Sender: TObject);
    procedure ExecNowMIClick(Sender: TObject);
    procedure VSTKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure VSTPaintText(Sender: TBaseVirtualTree;
      const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
      TextType: TVSTTextType);
    procedure ToggleEntriesActivityMIClick(Sender: TObject);
    procedure SpTBXPopupMenu1Popup(Sender: TObject);
    procedure ShowHintsBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  strict private
    { fields }
    ElapsedTime: Cardinal;
    FinishedTime: Cardinal;
    FPause,
    FFinished: Boolean;
    FinishingEntry: PVirtualNode;
    FHiddenOnStartup: Boolean;
  protected
    { properties and their setters and getters }
    function GetCurrentEntry: PVirtualNode;                     
    function GetEntryData: PStartupEntry;                  
    procedure SetFinished(const Value: Boolean);
    property CurrentEntryNode: PVirtualNode read GetCurrentEntry;
    property CurrentEntry: PStartupEntry read GetEntryData;
    property Finished: Boolean read FFinished write SetFinished;
  protected
    { setters and getters for public properties }
    procedure SetPause(const Value: Boolean);
  protected
    { methods }
    procedure Start(node: PVirtualNode; DeleteNode: Boolean);
//    procedure Finished;
    procedure EvaluateEntry(Node: PVirtualNode);
    procedure Execute(e: PStartupEntry);
  public
    { public methods }
    procedure Delete(node: PVirtualNode);
  public
    { public properties }
    property Pause: Boolean read FPause write SetPause;
  end;

  TIdleEntriesThread = class(TThread)
    protected
      procedure Execute; override;  
  end;

var
  StartForm: TStartForm;

implementation

uses
  Statics, ShellAPI, StartupAceSettings, SysUtils, StUpEntrysLoader, Math,
  TlHelp32, MiniBrowserWnd;

{$R *.dfm}

function ExtractFilePathW(FileName: WideString): WideString;
var
  I: Integer;
begin
  for I := Length(FileName) - 1 downto 1 do
  begin
    if copy(FileName, I, 1) = '\' then
    begin
      Result := copy(FileName, 1, I - 1);
      exit;
    end;
  end;
end;

procedure TStartForm.ConfigIClick(Sender: TObject);
begin
  Pause := True;
  ShellExecuteW(handle, 'open', PWChar(WideString(ParamStr(0))), '',
    PWChar(ExtractFilePathW(ParamStr(0))), SW_SHOWNORMAL);
end;

procedure TStartForm.DelayTimerElapsed(Sender: TObject);
var
  e: PStartupEntry;
  n: PVirtualNode;
  label notfinished;
begin
  Inc(ElapsedTime);
  ElapsedTimeLabel.Caption := IntToStr(ElapsedTime);
  DelayTimer.Enabled := false;
  // update list view so it displays the changed pies
  VST.Invalidate;
  Application.ProcessMessages;
  // loop through the list view items
  n := VST.GetFirst;
  while n <> nil do
  begin
    e := VST.GetNodeData(n);
    if e.Delay - ElapsedTime = 0 then
    begin
      // entry shall be started right this second; so do it now
      // checks process list if neccecary and simply ignores deactivated entries
      Start(n, false);
      n := VST.GetNext(n);
    end
    else if Finished then
    begin
      if n = FinishingEntry then
        // program in finishing stage and entry is the finishing entry;
        // look after the next one, if there has something changed
        n := VST.GetNext(n)
      else if e.Active then
      begin
        if Integer(e.Delay) - Integer(ElapsedTime) < 0 then
        begin
          // entry has already been started, remove it
          n := VST.GetNext(n);
          Delete(VST.GetPrevious(n));
        end
        else
        begin
          // program in finishing stage, entry shall be started in future
          // and is NOT the finishing entry but IS active;
          // there is a newly activated entry, process this one before finishing
          Finished := False;
          if not Pause then
            DelayTimer.Enabled := True;
          n := nil; // break loop
        end;
      end
      else
      begin
        // program in finishing stage, entry shall not be started;
        // continue countdown
        if not Pause then
          DelayTimer.Enabled := True;
        n := nil; // break loop
      end;
    end
    else
    begin
      // program not in finishing stage
      if e.Active then
      begin
        if Integer(e.Delay) - Integer(ElapsedTime) < 0 then
        begin
          // entry should have been started seconds earlier
          if e.WaitFor = '' then
          begin
            // remove entry, it has already been started
            n := VST.GetNext(n);
            if n = nil then
              Delete(VST.GetLast)
            else
              Delete(VST.GetPrevious(n));
          end
          else
          begin
            // entry is waiting for a process, look if it is time now
            Start(n, false);
            n := VST.GetNext(n);
          end;
        end
        else
        begin
          // the entry shall be started in future;
          // stop lurking through the list, just continue waiting
          if not Pause then
            DelayTimer.Enabled := True;     
          n := nil; // break loop
        end;
      end
      else
      begin
        // as the entry is deactivated threre do not seem to be any entries
        // left to start
        Finished := True;
        n := nil; // break loop
      end;
    end;
  end;
  
  // if there are no nodes (and entries) left
  // it's veeeery likely we can finish now
  if VST.GetFirst = nil then
    Finished := True;
end;

procedure TStartForm.ExecNowMIClick(Sender: TObject);
begin
  Start(VST.GetFirst, true);
end;

procedure TStartForm.ExecNowThisMIClick(Sender: TObject);
begin
  Start(VST.FocusedNode, true);
end;

procedure TStartForm.FormCreate(Sender: TObject);
begin
  VST.NodeDataSize := sizeof(TStartupEntry);
  TBXSwitcher1.Theme := 'Office2003';
end;

procedure TStartForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    VK_F2: begin
             ConfigIClick(self);
           end;
    Word('p'), Word('P'): begin
                            if ssCtrl in Shift then
                              PauseMIClick(self);
                          end;
  end;
end;

procedure TStartForm.FormShow(Sender: TObject);
begin
  self.OnShow := nil;         
  if not FHiddenOnStartup and TStartupAceSettings.StartMinimized then
  begin
    self.WindowState := wsMinimized;
    FHiddenOnStartup := True;
  end;
  StartupEntriesLoader.Load(VST, TStartupEntryEvent(nil),
    StartupEntriesLoader.StartupAce
    or StartupEntriesLoader.CurrentUser
    or StartupEntriesLoader.AllUsers
    or StartupEntriesLoader.Inactive);
  VST.SortTree(1, sdAscending);
  Pause := TStartupAceSettings.StartPaused;
  if VST.GetFirst <> nil then
  begin
    if PStartupEntry(VST.GetNodeData(VST.GetFirst)).Delay = 0 then
    begin
      Start(VST.GetFirst, false);
    end;
    DelayTimer.Enabled := True;
  end
  else
    Finished := True
end;

function TStartForm.GetCurrentEntry: PVirtualNode;
begin
  Result := VST.GetFirst;
end;

function TStartForm.GetEntryData: PStartupEntry;
begin
  Result := VST.GetNodeData(CurrentEntryNode);
end;

procedure TStartForm.EvaluateEntry(Node: PVirtualNode);
var
  e: PStartupEntry;
begin
  e := VST.GetNodeData(Node);
  if e.Delay - ElapsedTime <= 0 then
  begin
    Start(Node, false);
    if VST.GetNext(Node) <> nil then
      EvaluateEntry(VST.GetNext(Node));
  end
  else
  begin
    DelayTimer.Enabled := true;
  end;  
end;

procedure TStartForm.CancelMIClick(Sender: TObject);
begin
  Close;
end;

procedure TStartForm.PauseMIClick(Sender: TObject);
begin
  Pause := not Pause;
end;

procedure TStartForm.SetFinished(const Value: Boolean);
var
  e: PStartupEntry;
begin
  if FFinished <> Value then
  begin
    FFinished := Value;
    if FFinished then
    begin
      // add the 'closing window' entry
      FinishingEntry := VST.AddChild(nil);
      e := PStartupEntry(VST.GetNodeData(FinishingEntry));
      e^ := TStartupEntry.Create(FinishingEntry);
      e.TreeNode := FinishingEntry;
      e.Delay := ElapsedTime + 10;
      e.Caption := 'Finished, closing window';
      e.Path := 'REM';
      e.Arguments := 'Close Startup Ace';
      e.Active := True;
      e.SetCurrentAsOriginalValues;
      VST.SortTree(1, sdAscending);
      FinishedTime := ElapsedTime;
      ExecNowMI.Enabled := false;
      SkipMI.Enabled := false;
      PauseMI.Enabled := false;
      ElapsedTimeLabel.Caption := '--';
    end
    else
    begin
      // remove the entry and go on working...
      VST.DeleteNode(FinishingEntry);
      VST.SortTree(1, sdAscending);
      ExecNowMI.Enabled := True;
      SkipMI.Enabled := True;
      PauseMI.Enabled := True;
    end;                      
    DelayTimer.Enabled := True;
  end;
end;

procedure TStartForm.SetPause(const Value: Boolean);
begin
  FPause := Value;
  PauseMI.Checked := FPause;
  if not FPause then
  begin
    DelayTimer.Enabled := true;
  end;
end;

function IsProcessRunning(ProcessName: WideString): Boolean;
var
  SSHandle: THandle;
  ProcEntry: TProcessEntry32W;
  Continue: Boolean;
  osvi: OSVERSIONINFO;
begin  
  Result := False;
  osvi.dwOSVersionInfoSize := SizeOf(OSVERSIONINFO);
  GetVersionEx(osvi);
  if (osvi.dwMajorVersion > 4)
    or (osvi.dwPlatformId = VER_PLATFORM_WIN32_WINDOWS) then
  begin
    // Windows 9x/ME/2000/XP and higher, but not NT4
    SSHandle := CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);  // create a
      // snapshot of running processes list
    try
      ProcEntry.dwSize := SizeOf(ProcEntry);
      Continue := Process32FirstW(SSHandle, ProcEntry); // grab first process
      // process data saved in ProcEntry
      while Continue do
      begin
        if WideString(ProcEntry.szExeFile) = ProcessName then
        begin
          // process names match
          Result := True;
          Continue := False;
        end
        else
        begin
          Continue := Process32NextW(SSHandle, ProcEntry);  // else grab next
        end;
      end;
    finally
      CloseHandle(SSHandle);  // free process list snapshot
    end;
  end;
end;

procedure TStartForm.Execute(e: PStartupEntry);
begin
{$IFNDEF DONOTEXECUTEENTRYS}
  ShellExecuteW(handle, 'open', PWideChar(e.Path),
    PWideChar(e.Arguments),
    PWideChar(ExtractFilePathW(e.Path)), SW_SHOWNORMAL);
{$ENDIF}
end;

procedure TStartForm.Start(node: PVirtualNode; DeleteNode: Boolean);
var
  e: PStartupEntry;
begin
  e := VST.GetNodeData(node);
  if e.Active then
  begin
    if e.WaitFor <> '' then
    begin
      if IsProcessRunning(e.WaitFor) then
      begin
        Execute(e);
        e.WaitFor := '';
      end;
    end
    else
      if (e.Path = 'REM') and (e.Arguments = 'Close Startup Ace') then
        Close // close app when encountering finishing-entry
      else
        Execute(e);
  end;
  if DeleteNode then
    Delete(node);
end;

procedure TStartForm.ToggleEntriesActivityMIClick(Sender: TObject);
var
  e: PStartupEntry;
begin
  e := VST.GetNodeData(VST.FocusedNode);
  e.Active := not e.Active;
  if not e.Active then
  begin
    // if the entry becomes re-deactived reset the 'virtual' delay
    if not (e.Delay = e.OriginalDelay) then
      e.Delay := e.OriginalDelay;
  end;
  // save w/o deleting the old entry because its still the same entry
  e.Save(False, True);
  if e.Active and (e.Delay < ElapsedTime) then
    e.Delay := ElapsedTime + e.Delay;
  VST.SortTree(1, sdAscending);
  VST.InvalidateNode(VST.FocusedNode);
end;

procedure TStartForm.ShowHintsBtnClick(Sender: TObject);
begin
  TMiniBrowserWindow.Show('Hints',
    ExtractFilePathW(ParamStr(0)) + '\doc\startupformhints.htm');
end;

procedure TStartForm.Delete(node: PVirtualNode);
begin
  if node = VST.GetFirst then
  begin
    VST.DeleteNode(node);
    if VST.GetFirst = nil then
      Finished := True
  end
  else
  begin
    VST.DeleteNode(node);
  end;
end;

procedure TStartForm.SkipMIClick(Sender: TObject);
begin
  Delete(VST.GetFirst);
end;

procedure TStartForm.SkipThisMIClick(Sender: TObject);
begin
  Delete(VST.FocusedNode);
end;

procedure TStartForm.SpTBXPopupMenu1Popup(Sender: TObject);
var
  e: PStartupEntry;
begin
  if VST.FocusedNode <> nil then
  begin
    e := VST.GetNodeData(VST.FocusedNode);
    SkipThisMI.Enabled :=
      not (Finished and (VST.FocusedNode = FinishingEntry));
    ToggleEntriesActivityMI.Enabled :=
      e.Editable
      and not (Finished and (VST.FocusedNode = FinishingEntry));
    if e.Active then
      ToggleEntriesActivityMI.Caption := 'De&activate this startup entry'
    else
      ToggleEntriesActivityMI.Caption := 'Re&activate this startup entry';
  end;
end;

procedure TStartForm.VSTBeforeItemErase(Sender: TBaseVirtualTree;
  TargetCanvas: TCanvas; Node: PVirtualNode; ItemRect: TRect;
  var ItemColor: TColor; var EraseAction: TItemEraseAction);
var
  e: PStartupEntry;
begin
  e := Sender.GetNodeData(Node);
  if Sender.GetFirst = Node then
  begin
    ItemColor := $E0FFFF;
    EraseAction := eaColor;
  end
  else if (e.Delay - ElapsedTime <= 10) and e.Active then
  begin
    ItemColor := cl3DLight;
    EraseAction := eaColor;
  end;
end;

procedure TStartForm.VSTCompareNodes(Sender: TBaseVirtualTree; Node1,
  Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
begin
  case Column of
    1:
    begin
      if PStartupEntry(Sender.GetNodeData(Node1)).Active then
      begin
        if PStartupEntry(Sender.GetNodeData(Node2)).Active then
        begin
          Result := PStartupEntry(Sender.GetNodeData(Node1))^.Delay -
            PStartupEntry(Sender.GetNodeData(Node2))^.Delay;
        end
        else
        begin
          Result := -1;
        end;
      end
      else
      begin
        if PStartupEntry(Sender.GetNodeData(Node2)).Active then
        begin
          Result := 1;
        end
        else
        begin
          Result := PStartupEntry(Sender.GetNodeData(Node1))^.Delay -
            PStartupEntry(Sender.GetNodeData(Node2))^.Delay;
        end;
      end;
    end;
  end;
end;

procedure TStartForm.VSTFreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
var
  data: PStartupEntry;
begin
  data := Sender.GetNodeData(Node);
  data^.Free;
  data^.Caption := '';
end;

procedure TStartForm.VSTGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
  Column: TColumnIndex; TextType: TVSTTextType; var CellText: WideString);
var
  e: PStartupEntry;
begin
  try
    e := VST.GetNodeData(Node);
    case Column of
      0: CellText := e.Caption;
      1:
        if e.Delay = e.OriginalDelay then
          CellText := IntToStr(e.Delay)
        else
          CellText := '(' + IntToStr(e.Delay) + ')';
      2:
        if Sender.Selected[Node]
          or ((e.Delay - ElapsedTime <= 10) and e.Active)
          or ((Integer(e.Delay) - Integer(ElapsedTime) < 0) and (e.WaitFor <> ''))
          or (VST.GetFirst = Node)
          then
        begin
          if e.Active then
            if e.Delay - ElapsedTime = 0 then
              // entry is beeing started
              CellText := 'Is beeing started...'
            else if (Integer(e.Delay) - Integer(ElapsedTime) < 0 ) and (e.WaitFor <> '') then
              // entry is waiting for another process
              CellText := 'Waiting for process ' + e.WaitFor + ' to start...'
            else
              // entry is waiting for delay to elapse
              CellText := IntToStr((e.Delay - ElapsedTime)) + ' seconds'
          else
            CellText := 'Deactivated';
        end
        else CellText := '';
    end;
  except
    ;
  end;
end;

procedure TStartForm.VSTKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  r: TRect;
  p1: TPoint;
  p2: TPoint;
  n: PVirtualNode;
begin
  case Key of
    VK_SPACE: begin
                if ssShift in Shift then
                  Start(CurrentEntryNode, true)
                else
                  Start(VST.FocusedNode, true);
              end;
    VK_DELETE: begin
                 if VST.FocusedNode <> nil then
                 begin
                  n := VST.GetNext(VST.FocusedNode);
                  if n = nil then
                    n := VST.GetPrevious(VST.FocusedNode);
                  if ssShift in Shift then
                    Delete(CurrentEntryNode)
                  else
                    Delete(VST.FocusedNode);
                  VST.FocusedNode := n;
                  VST.Selected[n] := True;
                  VST.SetFocus;
                 end;
               end;
    VK_RETURN: begin
                 r := VST.GetDisplayRect(VST.FocusedNode, 0, true);
                 p1 := VST.ClientToScreen(r.TopLeft);
                 p2 := VST.ClientToScreen(r.BottomRight);
                 VST.PopupMenu.Popup(p1.X + 10, p2.Y);
               end;
  end;
end;

procedure TStartForm.VSTPaintText(Sender: TBaseVirtualTree;
  const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  TextType: TVSTTextType);
var
  e: PStartupEntry;
  NH: Integer;
  Left: Integer;
begin
  e := Sender.GetNodeData(Node);
  if e.Active then
    TargetCanvas.Font.Color := clWindowText
  else
    TargetCanvas.Font.Color := clGrayText;
  if Sender.Selected[Node] or (e.Delay - ElapsedTime <= 10) and e.Active then
  begin
    Sender.NodeHeight[Node] :=
      (Sender as TVirtualStringTree).DefaultNodeHeight * 2;
    if (e.Delay - ElapsedTime <= 10) and e.Active then
    begin
      NH := Sender.NodeHeight[Node];
      Left := (Sender as TVirtualStringTree).GetDisplayRect(Node, 2, false).Left;
      TargetCanvas.Pen.Color := clInactiveCaption;
      TargetCanvas.Brush.Color := clGradientInactiveCaption;
      if e.Delay - ElapsedTime = 10 then
        TargetCanvas.Ellipse(Left + 2,2,Left + NH-2,NH-2)
      else if e.Delay - ElapsedTime >= 8 then
        TargetCanvas.Pie(Left + 2,2, Left + NH-2,NH-2,
          Left + Ceil(1 + (NH/2 - NH/2 * Tan(Pi - (e.Delay-ElapsedTime)*Pi/5))), 0,
          Left + 1 + NH div 2, 0)
      else if e.Delay - ElapsedTime >= 6 then
        TargetCanvas.Pie(Left + 2,2,Left + NH-2,NH-2,
          Left + Ceil(1 + (NH/2 - NH/2 * Tan((e.Delay-ElapsedTime)*Pi/5 - Pi))), NH,
          Left + 1 + NH div 2, 0)
      else if e.Delay - ElapsedTime = 5 then
        TargetCanvas.Pie(Left + 2,2,Left + NH-2,NH-2,
          Left + 1 + NH div 2, NH,
          Left + 1 + NH div 2, 0)
      else if e.Delay - ElapsedTime >= 3 then
         TargetCanvas.Pie(Left + 2,2,Left + NH-2,NH-2,
          Left + Ceil(1 + (NH/2 + NH/2 * Tan(Pi - (e.Delay-ElapsedTime)*Pi/5))), NH,
          Left + 1 + NH div 2, 0)
      else if e.Delay - ElapsedTime > 0 then
        TargetCanvas.Pie(Left + 2,2,Left + NH-2,NH-2,
          Left + Ceil(1 + (NH/2 + NH/2 * Tan((e.Delay-ElapsedTime)*Pi/5))), 0,
          Left + 1 + NH div 2, 0);
    end;             
  end
  else
  begin
    Sender.NodeHeight[Node] := (Sender as TVirtualStringTree).DefaultNodeHeight;
  end;
end;

{ TIdleEntriesThread }

procedure TIdleEntriesThread.Execute;
begin
  inherited;

end;

end.
