(*
StartupAce Statics.pas Unit (several static classes)
Copyright (C) 2006  Jakob Reschke

This software is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this software; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*)

unit Statics;

interface

uses
  VirtualTrees, StartupEntry;

type
  RegistryKeys = class
    const Run = 'Software\Microsoft\Windows\CurrentVersion\Run';
    const StartupAce = Run + '\Startup Ace';
  end;

  StartupAceStartupEntries = class
    class procedure RegisterStartup; static;
    class procedure UnRegisterStartup; static;
  end;

implementation

uses
  Registry, Windows, Forms;

{ StartupAceAutostartEntries }

class procedure StartupAceStartupEntries.RegisterStartup;
var
  reg: TRegistry;
  correctCommand: String;
begin
  reg := TRegistry.Create;
  try
    reg.RootKey := HKEY_CURRENT_USER;
    if reg.OpenKey(RegistryKeys.Run, true) then
    begin
      correctCommand :=  '"' + ParamStr(0) + '" start';
      if not reg.ValueExists('StartupAce') or
        (reg.ReadString('StartupAce') <> correctCommand) then
      begin
        MessageBoxW(Application.Handle,
          'It is neccecary to create a normal startup entry in the registry ' +
          'for Startup Ace itself. Startup Ace must be run at startup so' +
          ' it can execute its entries.'#10#13 +
          'Some security software tools might warn you about this' +
          ' because it could also be malware that wants to be started' +
          ' with Windows. '#10#13#10#13 +
          'If you want Startup Ace to work, you have to allow that the ' +
          'entry is written.', 'Important Information',
          MB_OK or MB_ICONINFORMATION);
        reg.WriteString('StartupAce', correctCommand);
      end;
    end
    else
    begin
      MessageBoxW(Application.Handle,
        'Cannot access the Windows startup registry key!',
        'Error',
        MB_OK or MB_ICONERROR);
    end;
  finally
    reg.Free;
    reg.CloseKey;
  end;
end;

class procedure StartupAceStartupEntries.UnRegisterStartup;
var
  reg: TRegistry;
begin
  reg := TRegistry.Create;
  try
    reg.RootKey := HKEY_CURRENT_USER;
    if reg.OpenKey(RegistryKeys.Run, false) then
      reg.DeleteKey('StartupAce')
    else
       MessageBoxW(Application.Handle,
        'Cannot access the Windows startup registry key!',
        'Error',
        MB_OK or MB_ICONERROR);
  finally
    reg.CloseKey;
    reg.Free;
  end;

end;

end.
