(*
StartupAce Version.pas Unit
(Run time Startup Ace version information)
Copyright (C) 2006, 2007  Jakob Reschke

This software is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this software; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*)

unit Version;

interface

uses Windows;

type
  StartupAceVersion = class
  private type
    TGetVersionNumber = function(Operation: TVSFixedFileInfo): Integer;
  public
    const PublishDate = '2007-05-25';
    class function Major: Integer; static;
    class function Minor: Integer; static;
    class function Internal: Integer; static;
  private
    class function GetVersionNumber(Operation: TGetVersionNumber): Integer; static;
  end;

implementation

class function StartupAceVersion.GetVersionNumber(Operation: TGetVersionNumber): Integer;
var
  VersionInfoSize, VersionInfoValueSize, Zero: DWord;
  VersionInfo, VersionInfoValue: Pointer;
begin
  Result := 0;
  VersionInfoSize := GetFileVersionInfoSizeW(
    PWChar(WideString(ParamStr(0))), Zero);
  if VersionInfoSize = 0 then Exit;
  GetMem(VersionInfo, VersionInfoSize);
  try
    if GetFileVersionInfoW(PWChar(WideString(ParamStr(0))), 0,
      VersionInfoSize, VersionInfo)
      and VerQueryValueW(VersionInfo, '\', VersionInfoValue,
        VersionInfoValueSize) then
    begin
      Result := Operation(TVSFixedFileInfo(VersionInfoValue^));
    end;
  finally
    FreeMem(VersionInfo);
  end;
end;
        
class function StartupAceVersion.Major: Integer;
  function GetBuildNumber(fi: TVSFixedFileInfo): Integer;
  begin
    Result := HiWord(fi.dwFileVersionMS);
  end;
begin
  Result := StartupAceVersion.GetVersionNumber(@GetBuildNumber);
end;

class function StartupAceVersion.Minor: Integer;
  function GetBuildNumber(fi: TVSFixedFileInfo): Integer;
  begin
    Result := LoWord(fi.dwFileVersionMS);
  end;
begin
  Result := StartupAceVersion.GetVersionNumber(@GetBuildNumber);
end;

class function StartupAceVersion.Internal: Integer;
  function GetBuildNumber(fi: TVSFixedFileInfo): Integer;
  begin
    Result := LoWord(fi.dwFileVersionLS);
  end;
begin
  Result := StartupAceVersion.GetVersionNumber(@GetBuildNumber);
end;

end.
