(*
StartupAce paths.pas Unit (static class TPath with file path handling methods)
Copyright (C) 2006  Jakob Reschke

This software is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this software; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*)

unit paths;

interface

uses
  Windows;

function GetSpecialFolder(CSIDL: Integer): WideString;

type
  TPath = class
    class function Exists(Path: WideString): Boolean;
    class function Expand(Path: WideString): WideString;
    class function ExtractPath(const FileName: WideString): WideString; overload;
    class function ExtractPath(const FileName: WideString;
      PathDelimiter: WideChar): WideString; overload;
  end;

implementation

uses
  SysUtils, ShlObj, ActiveX, JSysDirs;

function GetSpecialFolder(CSIDL: Integer): WideString;
var
  PIIdL: PItemIDList;
  Path: array [0..MAX_PATH] of WChar;
  Allocator: IMalloc;
begin
  SHGetSpecialFolderLocation(0, CSIDL, PIIdL);
  SHGetPathFromIDListW(PIIdL, Path);
  if Succeeded(SHGetMalloc(Allocator)) then
    Allocator.Free(PIIdL);
  Result := Path;
end;

class function TPath.Exists(Path: WideString): Boolean;
begin
  Result := FileExists(Path) or DirectoryExists(Path);
end;

{$REGION 'Author: Mathias Simmack'}
  function ExpandEnvStr(const szInput: WideString): WideString;
  const
    MAXSIZE = 32768; // laut PSDK sind 32k das Maximum
  begin
    SetLength(Result,MAXSIZE);
    SetLength(Result,ExpandEnvironmentStringsW(pWideChar(szInput),
      @Result[1],length(Result)));
  end;
{$ENDREGION}

class function TPath.Expand(Path: WideString): WideString;
var
  buf: array [0..MAX_PATH] of WChar;
  l: Integer;
begin
  if not Exists(Path) then
  begin
    Path := ExpandEnvStr(Path);
    if not TPath.Exists(Result) then
    begin
      l := Length(Path);
      if Path[l] = #0 then SetLength(Path, l - 1);

      GetWindowsDirectoryW(buf, MAX_PATH);
      Result := WideString(buf) + '\' + Path;
      if TPath.Exists(Result) then Exit;
      Result := Result + '.exe';
      if TPath.Exists(Result) then Exit;

      GetSystemDirectoryW(buf, MAX_PATH);
      Result := WideString(buf) + '\' + Path;
      if TPath.Exists(Result) then Exit;
      Result := Result + '.exe';
      if TPath.Exists(Result) then Exit;
      Result := Path;
    end;
  end else Result := Path;
end;

class function TPath.ExtractPath(const FileName: WideString;
  PathDelimiter: WideChar): WideString;                            
var
  I: Integer;
begin
  Result := '';
  I := Length(FileName);
  while (Result = '') and (I > 0) do
  begin
    if FileName[I] = '\' then
      Result := Copy(FileName, 1, I);
    Dec(I);
  end;
end;

class function TPath.ExtractPath(const FileName: WideString): WideString;
begin
  Result := TPath.ExtractPath(FileName, WideChar('\'));
end;

end.
